import { transSize, randomStr } from '../../../utils/platform';
import { getCloudInstance, getDefaultUploadPath } from '../../../utils/tcb';

const ACTION = { CHOOSE_MEDIA: 'chooseMedia', CHOOSE_MESSAGE_FILE: 'chooseMessageFile' };

// 选择文件
const chooseFile = ({ action = 'CHOOSE_MEDIA', config, success, fail }) => {
  let _config = { count: 9, maxDuration: 60 };

  switch (action) {
    case ACTION['CHOOSE_MEDIA']:
      _config = {
        mediaType: ['image'],
        sourceType: ['album'],
        ...config,
      };
      break;
    case ACTION['CHOOSE_MESSAGE_FILE']:
      _config = {
        type: 'all',
        ...config,
      };
      break;
    default:
      _config = { ...config };
      break;
  }

  wx[ACTION[action]]({
    ..._config,
    success,
    fail,
  });
};

export const getName = (hf) => {
  const uuidReg = /[0-9a-f]{8}([0-9a-f]{4}){3}[0-9a-f]{12}-/;
  const lastIndex = String(hf).lastIndexOf('/');
  const name = String(hf).slice(lastIndex + 1);
  const label = name.replace(uuidReg, '');
  return label;
};

// 批量上传文件前置事件
const handleBefore = (files, uploadInstance) => {
  const { count: maxCount, maxSize } = uploadInstance.config;
  const previewFile = uploadInstance.getPreviewFile();
  if (previewFile.length + files.length > maxCount) {
    wx.showToast({
      title: `上传文件总数不能超过${maxCount}个`,
      icon: 'none',
      duration: 2000,
    });
    return false;
  }
  if (maxSize && files.some((f) => f.size > maxSize * 1024 * 1024)) {
    wx.showToast({
      title: `上传文件大小不能超过${maxSize}M`,
      icon: 'none',
      duration: 2000,
    });
    return false;
  }
  return true;
};

// 上传
export const handleUpload = async (files, uploadInstance, reload = false) => {
  if (!handleBefore(files, uploadInstance)) return;

  let shouldUploadToCos = true;
  if (typeof uploadInstance?.beforeUpload === 'function') {
    try {
      const mgr = wx.getFileSystemManager();
      const ret = await uploadInstance.beforeUpload({
        tempFilePaths: files.map((f) => f.path || f.tempFilePath),
        base64Uri: files.map((f) => {
          const content = mgr.readFileSync(f.path || f.tempFilePath);
          return `data:;base64,${wx.arrayBufferToBase64(content)}`;
        }),
      });
      if (typeof ret === 'boolean') {
        shouldUploadToCos = ret;
      } else if (Array.isArray(ret) && ret.every((i) => typeof i === 'string')) {
        files = ret.map((path, idx) => {
          files[idx].path = path;
        });
      }
    } catch (err) {
      console.error('上传前处理函数抛错', err);
      shouldUploadToCos = false;
    }
  }
  if (!shouldUploadToCos) {
    return;
  }

  const initFiles = files.map((i) => ({
    ...i,
    percent: '0',
    cloudPath: null,
    name: i?.name || getName(i?.tempFilePath),
    filePath: i?.path || i?.tempFilePath || null,
    size: transSize(i?.size),
    status: 'pending',
  }));
  const previewFile = uploadInstance.getPreviewFile();
  let filelsList = reload ? previewFile : [...previewFile, ...initFiles];
  // 更新预览文件
  uploadInstance.updatePreviewFile(filelsList);
  return Promise.all(
    initFiles.map(async (tempFile) => {
      return new Promise(function (resolve) {
        handleUploadFile({
          uploadPath: uploadInstance.config.uploadPath,
          customUploadPath: uploadInstance.config.customUploadPath,
          _tempFile: tempFile,
          onSuccess: (res) => {
            filelsList = filelsList.map((i) => (i.filePath === res.filePath ? res : i));
            uploadInstance.updatePreviewFile(filelsList);
            uploadInstance.onSuccess(res);
            resolve(res);
          },
          onProgressUpdate: (res) => {
            filelsList = filelsList.map((i) => (i.filePath === res.filePath ? res : i));
            uploadInstance.updatePreviewFile(filelsList);
          },
          onFail: (res, error) => {
            filelsList = filelsList.map((i) => (i.filePath === res.filePath ? res : i));
            uploadInstance.updatePreviewFile(filelsList);
            uploadInstance.onFail(res, error);
            resolve(res);
          },
        });
      });
    }),
  ).then((res) => {
    const result = filelsList.map((i) => res.find((j) => j.filePath === i.filePath) || i);

    uploadInstance.onComplete(result);
  });
};

//
const sanitizeFilename = (filename, replacement = '') => {
  return (
    filename
      // 移除非法字符
      // eslint-disable-next-line no-control-regex
      .replace(/[<>.:"\\|?*\x00-\x1F]/g, replacement)
      // 处理Windows保留文件名（CON, PRN等）
      .replace(/^(CON|PRN|AUX|NUL|COM[1-9]|LPT[1-9])(\..*)?$/i, replacement)
      // 移除开头和结尾的点,空格,反斜杠（Windows不允许）
      .replace(/^[. /]+/, '')
      .replace(/[. /]+$/, '')
      // 可选：限制长度
      .slice(0, 255)
  );
};
// 上传到云存储
const handleUploadFile = async ({
  uploadPath = 'weda-uploader',
  _tempFile,
  onSuccess,
  onProgressUpdate,
  onFail,
  customUploadPath,
}) => {
  const tempFile = {
    ..._tempFile,
  };
  let _uploadPath = await getDefaultUploadPath(uploadPath);
  if (customUploadPath) {
    _uploadPath = `${_uploadPath}/${sanitizeFilename(customUploadPath, '')}`;
  }
  const filenameRegex = /[^a-zA-Z0-9\u4e00-\u9fff-*!_.]/g;
  const cloudPath = `${_uploadPath}/${randomStr()}-${tempFile.name?.replace(filenameRegex, '-')}`;
  try {
    const tcb = await getCloudInstance();
    const uploadTask = await tcb.uploadFile({
      cloudPath: cloudPath,
      filePath: tempFile.path || tempFile.tempFilePath,
      success(res) {
        tempFile.cloudPath = res.fileID;
        tempFile.percent = 100;
        tempFile.status = 'success';
        if (onSuccess) {
          onSuccess(tempFile);
        }
      },
      fail(e) {
        tempFile.status = 'fail';
        wx.showModal({
          title: '上传失败，请重试',
          content: e.message,
          showCancel: false,
        });
        onFail(tempFile, e);
      },
    });
    if (typeof uploadTask.onProgressUpdate === 'function') {
      uploadTask.onProgressUpdate((res) => {
        tempFile.percent = String(res.progress);
        tempFile.status = res.progress == 100 ? 'success' : 'uploading';
        tempFile.uploaded = transSize(res.totalBytesSent);
        if (onProgressUpdate) {
          onProgressUpdate(tempFile);
        }
      });
    } else {
      tempFile.percent = '100';
      tempFile.status = 'success';
      tempFile.cloudPath = uploadTask.fileID;
      onSuccess(tempFile);
    }
  } catch (e) {
    tempFile.status = 'fail';
    wx.showModal({
      title: '上传失败，请重试',
      content: e.message,
      showCancel: false,
    });
    if (onFail) {
      onFail(tempFile, e);
    }
  }
};

export const upload = (uploadInstance) => {
  const { config, onSuccess, onFail } = uploadInstance;
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const { action, disabled, maxSize, ..._config } = config;
  if (disabled) return;
  const success = (res) => {
    const result = handleUpload(res.tempFiles, uploadInstance);
    onSuccess(result);
  };
  const fail = (e) => {
    onFail(e);
  };

  chooseFile({ action, config: _config, success, fail });
};

export const initUploadInstance = ({ action, config, previewFile }, _this) => {
  let uploadInstance = {
    action,
    config: { disabled: false, maxSize: 500, count: 9, maxDuration: 60, ...config },
    previewFile,
    updatePreviewFile(files) {
      _this.setData({ files });
    },
    getPreviewFile() {
      const { files } = _this.data;
      return files;
    },
    onComplete(result) {
      const cloudPathList = result.filter((i) => i.cloudPath).map((j) => j.cloudPath);
      _this.setData({
        files: result,
        urls: cloudPathList,
        cloudFile: cloudPathList,
      });
      let _value = cloudPathList;
      if (_this.data.single) {
        _value = cloudPathList[0] ?? '';
      }
      _this.handleChange(_value);
    },
    onSuccess(res) {
      _this.triggerEvent('success', {
        value: res.cloudPath,
        file: res,
      });
    },
    onFail(e) {
      _this.triggerEvent('error', e);
    },
    beforeUpload: _this.data.callbacks?.beforeUpload,
  };

  return uploadInstance;
};
