import destr from '../../../utils/destr';
import { callDataSourceApi, callWedaApi } from '../../../utils/tcb';
import { arrayToMap } from '../../../utils/tool';
import lodashGet from 'lodash.get';
import deepEqual from '../../../utils/deepEqual';
import { getDefaultQuery } from '../../../utils/getModelParams';
import { isFormatNeedFetch, isFormatWithFilterable } from './formats-util';
import { textToString } from '../../../utils/platform';

const STATUS_CLOSED = 0;
const STATUS_LOADING = 1;
const STATUS_FAILED = 2;
const SEARCH_STATUS_EMPTY = 3;
const STATUS_EMPTY = -1;
Component({
  options: {
    virtualHost: true,
    pureDataPattern: /^__/,
  },
  properties: {
    placeholder: {
      type: String,
      value: '请选择',
    },
    disabled: {
      type: Boolean,
      value: false,
    },
    range: {
      type: Array,
      value: [],
    },
    defaultValue: {
      type: null,
    },
    format: {
      type: String,
      value: '',
    },
    dataSourceName: {
      type: String,
      value: '',
    },
    viewId: {
      type: String,
      value: '',
    },
    primaryField: {
      type: String,
      value: '',
    },
    enumName: {
      type: String,
      value: '',
    },
    where: {
      type: Array,
      value: [],
    },
    allPickerShow: {
      type: Boolean,
      value: false,
    },
    version: {
      type: String,
      value: '',
    },
    readOnly: {
      type: Boolean,
      value: false,
    },
    ignoreCase: {
      type: Boolean,
      value: true,
    },
    supportManyRelated: {
      type: Boolean,
      value: false,
    },
    queryCondition: {
      type: null,
      value: [],
    },
    sorter: {
      type: Array,
      value: [],
    },
    multiple: {
      type: Boolean,
      value: false,
    },
    searchable: {
      type: Boolean,
      value: true,
    },
    filterable: {
      type: Boolean,
      value: false,
    },
  },
  data: {
    selectRange: [],
    value: null,
    loadStatus: 0,
    option: [],
    _queryCondition: {},
    queryParam: { select: { $master: true } },
    currentPageNo: 1,
    _filterable: false,
    hasNextPage: true,
  },
  lifetimes: {
    attached() {
      const { range, filterable, format, defaultValue } = this.properties;
      const label = this.getLabels(defaultValue, range);

      this.setData({
        option: range,
        _filterable: format && isFormatWithFilterable(format) ? true : filterable,
        value: [].concat(defaultValue),
        displayValue: label,
      });
      this._initFetchData({ searchValue: '' });
    },
    detached() {
      const id = this.data.timeId;
      if (id !== '') {
        clearTimeout(id);
      }
    },
  },
  observers: {
    defaultValue: function (defaultValue) {
      const _value = [].concat(defaultValue);
      if (deepEqual(_value, this.data.value)) {
        return;
      }
      this.setData({
        value: _value,
      });
      if (_value.filter((i) => textToString(i)).length) {
        this._initFetchData({ searchValue: '' });
      }
    },
    'value,range': function (value, range) {
      let _option = this.data.option;
      if (!isFormatNeedFetch(this.data.format)) {
        _option = range;
      }
      const label = this.getLabels(value, _option, this.data.displayValue);
      const checkRange = this.getChecks(value, _option);
      // 默认
      this.setData({
        option: checkRange,
        displayValue: label,
      });
    },
    enumName: function (enumName) {
      if (this.properties.format === 'x-enum' && enumName) {
        this._fetchEnumData();
      }
    },
    'where,queryCondition,sorter': function (where, queryCondition, sorter) {
      const _queryParam = getDefaultQuery({
        supportManyRelated: this.data.supportManyRelated,
        queryCondition,
        where,
        sorter,
      });
      if (deepEqual(_queryParam, this.data.queryParam)) {
        return;
      }

      this.setData({ queryParam: _queryParam }, () => {
        this._initFetchData({ queryParam: _queryParam });
      });
    },
    format: function (format) {
      if (isFormatWithFilterable(format)) {
        // 关联关系，自定义查询规则，不走前端过滤
        this.setData({
          _filterable: true,
        });
      }
    },
  },
  methods: {
    onClosePicker: function () {
      this.setData({
        allPickerShow: false,
      });
      this._initFetchData({ searchValue: '' });
    },
    onOpenPicker: function () {
      if (this.data.disabled === true) return;
      this.setData({
        allPickerShow: true,
      });
      this.triggerEvent('focus');
    },
    // 最终选择的选项
    onSelectPicker: function (e) {
      if (this.data.disabled === true) return;
      if (this.data.multiple) {
        const value = e.detail.map((v) => v.value);
        const options = e.detail.map((v) => ({ label: v.label, value: v.value, extra: { ...v } }));

        this.setData({ value, allPickerShow: false });
        this.onChange({ detail: { value, context: { options } } });
      } else {
        this.setData({ value: [].concat(e.detail.value), allPickerShow: false });
        this.onChange({ detail: { value: e.detail.value, context: { option: e.detail } } });
      }
    },
    // 获取数据列表：关联关系和主子明细
    _fetchData: async function ({ queryParam = this.data.queryParam, searchValue = '', pageNo = 1, _init = false }) {
      const { dataSourceName, viewId, primaryField } = this.data;
      if (!dataSourceName) return;
      this.setData({ loadStatus: STATUS_LOADING });
      //   默认筛选条件
      let _filter = destr(this.data.queryParam.filter) ?? {};
      // 根据搜索值查询
      if (searchValue) {
        const search = this.data.ignoreCase ? '$search_ci' : '$search';
        _filter = {
          where: {
            $and: [_filter.where, { $and: [{ [primaryField]: { [`${search}`]: searchValue } }] }],
          },
        };
      }
      const records = _init ? [] : this.data.option;
      let pageSize = 50;
      let res = await callDataSourceApi({
        dataSourceName: dataSourceName,
        viewId: viewId,
        methodName: 'wedaGetRecordsV2',
        params: {
          ...queryParam,
          filter: _filter,
          pageNumber: pageNo,
          pageSize: pageSize,
        },
      });
      const results = res?.records.map((item) => ({
        ...item,
        label: item[primaryField],
        value: item._id,
      }));

      let finalRecords = records.concat(results || []);
      finalRecords = this.getUniqueOption(finalRecords);
      const { value } = this.data;
      const unchecked = value?.filter((i) => i && !finalRecords.find((j) => i === j.value));
      // 跨页勾选回显示
      if (unchecked?.length) {
        const uncheckedItem = await this.fetchItem(unchecked);
        uncheckedItem?.forEach((i) => {
          if (i) {
            finalRecords.unshift(i);
          }
        });
      }
      const label = this.getLabels(value, finalRecords);
      const stringRange = this.getChecks(value, finalRecords);
      let status = STATUS_CLOSED;
      if (!results) {
        status = STATUS_FAILED;
      } else if (finalRecords.length === 0) {
        status = SEARCH_STATUS_EMPTY;
      }
      // 默认
      this.setData({
        option: stringRange,
        displayValue: label,
        loadStatus: status,
        currentPageNo: pageNo,
        hasNextPage: stringRange?.length < res?.total,
      });
      this.triggerEvent('updateSelect', { value: value, option: stringRange });
    },
    _childFetchData: function (data) {
      if (this.data.hasNextPage) {
        const { pageNo, searchValue } = data.detail;
        this._fetchData({ pageNo, searchValue });
      }
    },
    getLabels: function (values, options, _preLabel) {
      // 缓存展示值 _preLabel
      let labels = values;
      if (Array.isArray(values) && Array.isArray(options)) {
        const rm = arrayToMap(options, 'value');
        labels = values.map((d) => {
          const obj = rm[d];
          let item = lodashGet(obj, 'text') ?? lodashGet(obj, 'label') ?? _preLabel ?? d;
          return textToString(item);
        });
      }
      if (this.data.multiple) {
        return labels?.join(',');
      } else {
        return labels?.[0];
      }
    },
    getChecks: function (values, options) {
      const _values = Array.isArray(values) ? values : [];
      const _options = Array.isArray(options) ? options : [];
      const stringRange = _options.map((item) => {
        const obj = _values.find((el) => el === item.value);
        const check = !!obj;
        const value = String(item.value);
        return { ...item, value, check };
      });
      return stringRange;
    },
    fetchItem: async function (values) {
      const { queryParam, dataSourceName, primaryField } = this.data;
      const _filter = {
        where: {
          $and: [{ $or: values.map((i) => ({ _id: { $search: i } })) }],
        },
      };
      let data = await callDataSourceApi({
        dataSourceName: dataSourceName,
        methodName: 'wedaGetRecordsV2',
        params: {
          ...queryParam,
          filter: _filter,
          pageNumber: 1,
          pageSize: 200,
        },
      });
      return (
        data?.records?.map((item) => ({
          ...item,
          label: item[primaryField],
          value: item._id,
        })) || []
      );
    },
    // 获取通用选项集列表
    _fetchEnumData: async function () {
      let data = await callWedaApi({
        action: 'DescribeGeneralOptionsDetailList',
        data: {
          OptNameList: [this.properties.enumName],
          PageIndex: 1,
          PageSize: 10,
        },
      });

      const config = destr(data?.Items?.[0]?.Config) ?? [];
      const option = config.map((item) => {
        return {
          label: item.value,
          value: item.key,
          name: item.value,
        };
      });
      this.setData({
        selectRange: option.map((item) => item.label),
        option,
        loadStatus: option.length > 0 ? 0 : STATUS_EMPTY,
      });
    },
    getUniqueOption: function (option) {
      const optionMap = option.reduce((acc, item) => {
        if (!acc[item.value]) {
          acc[item.value] = item;
        }
        return acc;
      }, {});
      const uniqueOption = Object.values(optionMap)?.filter((item) => textToString(item?.value));
      return uniqueOption;
    },
    // 初始化搜索数据，包括属性变化/搜索值变化
    _initFetchData: function ({ queryParam = this.data.queryParam, searchValue = '' }) {
      clearTimeout(this.data.fetchTimed);
      if (!isFormatWithFilterable(this.data.format)) {
        return;
      }
      this.setData({ option: [], pageNo: 1, status: STATUS_LOADING });
      // eslint-disable-next-line rulesdir/no-timer
      this.data.fetchTimed = setTimeout(() => {
        this._fetchData({ queryParam, searchValue, _init: true });
      }, 200);
    },
    onChange(e) {
      this.triggerEvent('change', e.detail);
      this.triggerEvent('updateSelect', { value: e.detail.value, option: this.data.option });
    },
    onCancel(e) {
      this.triggerEvent('cancel', e.detail);
    },
    onSearch(event) {
      const { noEvent, value: searchValue } = event.detail;

      // 取消不触发搜索事件
      if (!noEvent) {
        this.triggerEvent('search', { value: searchValue });
      }

      this._initFetchData({ searchValue: event.detail.value });
    },
  },
});
